//////////////////////////////////////////////////////////////////////////////
//
// Module:
//  mmu.h
//
// Description:
//  This module implements prototype, defines and structures needed
//  by the mmu.c module of the MMULib.
//
// Author:
//  (c) 2004 Jouni 'Mr.Spiv' Korhonen
//
// Version:
//  v0.1 - 9-May-2004 Initial release version.
//
//////////////////////////////////////////////////////////////////////////////
//
// 
// below are the mmu.h defined "language" for creating custom
// MMU table setups.
//
//
//
//
// [GLOBALS]
//
//   GBL_DOMAIN, <domain>
//   GBL_AP, AP_RORO|AP_RW_NA|AP_RWRO|AP_RWRW
//   GBL_CB, NCNB|CNB|NCB|CB
//
// [LEVEL1]
//
//   LVL1,COARSE|FINE|FAULT,<start>,<end>,[LVL1_FLAGS],  or
//   LVL1,SECTION,<start>,<end>,<phys>,[LVL1_FLAGS],
//        LVL2,<start>,<end>,<phys>,LARGE|SMALL|TINY|FAULT,[LVL2_FLAGS]
//        LVL2,
//        LVL2,
//        ...
//   LVL1,...
//   MMU_END
//
// [LVL1_FLAGS]
//   LVL1_DOMAIN,<#>
//   LVL1_AP, AP_RORO|AP_RWNA|AP_RWRO|AP_RWRW
//   LVL1_CB, NCNB|CNB|NCB|CB
//
// [LVL2_FLAGS]
//   LVL2_CB, NCNB|CNB|NCB|CB
//   LVL2_AP0, AP_RORO|AP_RWNA|AP_RWRO|AP_RWRW
//   LVL2_AP1, AP_RORO|AP_RWNA|AP_RWRO|AP_RWRW
//   LVL2_AP2, AP_RORO|AP_RWNA|AP_RWRO|AP_RWRW
//   LVL2_AP3, AP_RORO|AP_RWNA|AP_RWRO|AP_RWRW
//

#ifndef _MMU_H_included
#define _MMU_H_included

#ifdef __cplusplus
extern "C" {
#endif

enum mmu_lang {
	MMU_DONE,
	GLOBAL,
	GBL_DOMAIN,
	GBL_AP,
	GBL_CB,
	
	LVL1,
	COARSE,
	FINE,
	SECTION,
	FAULT,
	
	LVL2,
	LARGE,
	SMALL,
	TINY,
	
	LVL1F_DOMAIN,
	LVL1F_AP,
	LVL1F_CB,
	
	LVL2F_CB,
	LVL2F_AP0,
	LVL2F_AP1,
	LVL2F_AP2,
	LVL2F_AP3
};

#define	LVL2F_AP	LVL2F_AP0

#define	AP_RORO	0x00    //
#define	AP_RONA	0x01    //
#define	AP_RWRO	0x02    //
#define	AP_RWRW	0x03    //

#define CB	0x0c    // cache & writeback enabled
#define CNB	0x08    // cache enabled, writeback disabled
#define NCB	0x04    // cache disabled, writeback enabled
#define NCNB	0x00    // cache & writeback disabled

#define	FAULT_LVL1	0x00000000
#define	COARSE_LVL1	0x00000011
#define	SECTION_LVL1	0x00000012
#define	FINE_LVL1	0x00000013

#define	FAULT_LVL2	FAULT_LVL1
#define	LARGE_LVL2	0x00000001
#define	SMALL_LVL2	0x00000002
#define	TINY_LVL2	0x00000003

//

#define	AP0_SHIFT	4
#define	AP1_SHIFT	6
#define	AP2_SHIFT	8
#define	AP3_SHIFT	10
#define	AP_SHIFT	AP0_SHIFT
#define SECTION_AP_SHIFT	10

#define	DOMAIN_SHIFT	5

//
//
//

#define	CACHE_INST_MSK	(1 << 12)
#define	CACHE_DATA_MSK	(1 << 2)
#define EXCEPT_LOW		0
#define EXCEPT_HIGH		1



//
//
//

int MMU_Init( unsigned long, int, unsigned long * );

void disableMMU( void );
void enableMMU( void );
void invalidateTLBs( void );
int disableAndFlushCaches( int );
void enableCaches( int );
void setTTB( unsigned long * );
unsigned long *getTTB( void );
void setPID( int );
void setDomain( int );
void modifyRS(int,int);
void invalidateCaches( void );
void invalidateDCache( void );
void invalidateICache( void );
void drainWB( void );
void setVectorLocation( int loc );

//
// Default BIOS alike MMU setups.. Except for
// memory mapped io section but you are not supposed
// to modify caching on that area anyways.
//

// 8M setup
#if 0
unsigned long _gp32_std_8M_mmu[] = {
  GBL_DOMAIN, 0xf,			// Define "global" domain
  LVL1,
    COARSE,0x00000000,0x000fffff,
    LVL2,0x00000000,0x0007ffff,0x0c800000,
      LARGE,LVL2F_AP,AP_RORO,LVL2F_CB,CNB,
  LVL1,
    COARSE,0x0c000000,0x0c800000,
    LVL2,0x0c000000,0x0c6fffff,0x0c000000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,CB,
    LVL2,0x0c700000,0x0c7fffff,0x0c700000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,NCNB,
  LVL1,
    SECTION,0x14000000,0xe9ffffff,0x14000000,LVL1F_AP,AP_RWRW,LVL1F_CB,NCNB,
  MMU_DONE	
};

// 32M setup

unsigned long _gp32_std_32M_mmu[] = {
  GBL_DOMAIN, 0xf,			// Define "global" domain
  LVL1,
    COARSE,0x00000000,0x000fffff,
    LVL2,0x00000000,0x0007ffff,0x0c800000,
      LARGE,LVL2F_AP,AP_RORO,LVL2F_CB,CNB,
  LVL1,
    COARSE,0x0c000000,0x0c800000,
    LVL2,0x0c000000,0x0c6fffff,0x0c000000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,CB,
    LVL2,0x0c700000,0x0c7fffff,0x0c700000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,NCNB,
  LVL1,
    COARSE,0x0c800000,0x0dffffff,
    LVL2,0x0c800000,0x0dffffff,0x0c800000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,CB,  LVL1,
    SECTION,0x14000000,0xe9ffffff,0x14000000,LVL1F_AP,AP_RWRW,LVL1F_CB,NCNB,
  MMU_DONE	
};

// 64M setup

unsigned long _gp32_std_64M_mmu[] = {
  GBL_DOMAIN, 0xf,			// Define "global" domain
  LVL1,
    COARSE,0x00000000,0x000fffff,
    LVL2,0x00000000,0x0007ffff,0x0c800000,
      LARGE,LVL2F_AP,AP_RORO,LVL2F_CB,CNB,
  LVL1,
    COARSE,0x0c000000,0x0c800000,
    LVL2,0x0c000000,0x0c6fffff,0x0c000000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,CB,
    LVL2,0x0c700000,0x0c7fffff,0x0c700000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,NCNB,
  LVL1,
    COARSE,0x0c800000,0x0dffffff,
    LVL2,0x0c800000,0x0dffffff,0x0c800000,
      SMALL,LVL2F_AP,AP_RWRW,LVL2F_CB,CB,  LVL1,
    SECTION,0x14000000,0xe9ffffff,0x14000000,LVL1F_AP,AP_RWRW,LVL1F_CB,NCNB,
  MMU_DONE	
};
#endif
#ifdef __cplusplus
}
#endif
#endif




